@@ -47,28 +47,28 @@
 #define TK_FOR                            47
 #define TK_FOREIGN                        48
 #define TK_FROM                           49
-#define TK_FULL                           50
-#define TK_FUNCTION                       51
-#define TK_GE                             52
-#define TK_GLOB                           53
-#define TK_GROUP                          54
-#define TK_GT                             55
-#define TK_HAVING                         56
-#define TK_ID                             57
-#define TK_IGNORE                         58
-#define TK_ILLEGAL                        59
-#define TK_IMMEDIATE                      60
-#define TK_IN                             61
-#define TK_INDEX                          62
-#define TK_INITIALLY                      63
-#define TK_INSERT                         64
-#define TK_INSTEAD                        65
-#define TK_INTEGER                        66
-#define TK_INTERSECT                      67
-#define TK_INTO                           68
-#define TK_IS                             69
-#define TK_ISNULL                         70
-#define TK_JOIN                           71
+#define TK_FUNCTION                       50
+#define TK_GE                             51
+#define TK_GLOB                           52
+#define TK_GROUP                          53
+#define TK_GT                             54
+#define TK_HAVING                         55
+#define TK_ID                             56
+#define TK_IGNORE                         57
+#define TK_ILLEGAL                        58
+#define TK_IMMEDIATE                      59
+#define TK_IN                             60
+#define TK_INDEX                          61
+#define TK_INITIALLY                      62
+#define TK_INSERT                         63
+#define TK_INSTEAD                        64
+#define TK_INTEGER                        65
+#define TK_INTERSECT                      66
+#define TK_INTO                           67
+#define TK_IS                             68
+#define TK_ISNULL                         69
+#define TK_JOIN                           70
+#define TK_JOIN_KW                        71
 #define TK_KEY                            72
 #define TK_LE                             73
 #define TK_LIKE                           74
@@ -87,40 +87,39 @@
 #define TK_ON                             87
 #define TK_OR                             88
 #define TK_ORDER                          89
-#define TK_PARTIAL                        90
-#define TK_PLUS                           91
-#define TK_PRAGMA                         92
-#define TK_PRIMARY                        93
-#define TK_RAISE                          94
-#define TK_REFERENCES                     95
-#define TK_REM                            96
-#define TK_REPLACE                        97
-#define TK_RESTRICT                       98
-#define TK_ROLLBACK                       99
-#define TK_ROW                            100
-#define TK_RP                             101
-#define TK_RSHIFT                         102
-#define TK_SELECT                         103
-#define TK_SEMI                           104
-#define TK_SET                            105
-#define TK_SLASH                          106
-#define TK_SPACE                          107
-#define TK_STAR                           108
-#define TK_STATEMENT                      109
-#define TK_STRING                         110
-#define TK_TABLE                          111
-#define TK_TEMP                           112
-#define TK_THEN                           113
-#define TK_TRANSACTION                    114
-#define TK_TRIGGER                        115
-#define TK_UMINUS                         116
-#define TK_UNCLOSED_STRING                117
-#define TK_UNION                          118
-#define TK_UNIQUE                         119
-#define TK_UPDATE                         120
-#define TK_USING                          121
-#define TK_VACUUM                         122
-#define TK_VALUES                         123
-#define TK_VIEW                           124
-#define TK_WHEN                           125
-#define TK_WHERE                          126
+#define TK_PLUS                           90
+#define TK_PRAGMA                         91
+#define TK_PRIMARY                        92
+#define TK_RAISE                          93
+#define TK_REFERENCES                     94
+#define TK_REM                            95
+#define TK_REPLACE                        96
+#define TK_RESTRICT                       97
+#define TK_ROLLBACK                       98
+#define TK_ROW                            99
+#define TK_RP                             100
+#define TK_RSHIFT                         101
+#define TK_SELECT                         102
+#define TK_SEMI                           103
+#define TK_SET                            104
+#define TK_SLASH                          105
+#define TK_SPACE                          106
+#define TK_STAR                           107
+#define TK_STATEMENT                      108
+#define TK_STRING                         109
+#define TK_TABLE                          110
+#define TK_TEMP                           111
+#define TK_THEN                           112
+#define TK_TRANSACTION                    113
+#define TK_TRIGGER                        114
+#define TK_UMINUS                         115
+#define TK_UNCLOSED_STRING                116
+#define TK_UNION                          117
+#define TK_UNIQUE                         118
+#define TK_UPDATE                         119
+#define TK_USING                          120
+#define TK_VACUUM                         121
+#define TK_VALUES                         122
+#define TK_VIEW                           123
+#define TK_WHEN                           124
+#define TK_WHERE                          125
